@@ -7,7 +7,6 @@ import (
 	"regexp"
 	"strconv"
 	"strings"
-	"syscall"
 	"time"
 	"unicode/utf8"
 
@@ -23,27 +22,13 @@ const (
 	defaultEscDelay = 100
 	escPollInterval = 5
 	offsetPollTries = 10
+	maxInputBuffer  = 10 * 1024
 )
 
 const consoleDevice string = "/dev/tty"
 
 var offsetRegexp *regexp.Regexp = regexp.MustCompile("(.*)\x1b\\[([0-9]+);([0-9]+)R")
 
-func openTtyIn() *os.File {
-	in, err := os.OpenFile(consoleDevice, syscall.O_RDONLY, 0)
-	if err != nil {
-		tty := ttyname()
-		if len(tty) > 0 {
-			if in, err := os.OpenFile(tty, syscall.O_RDONLY, 0); err == nil {
-				return in
-			}
-		}
-		fmt.Fprintln(os.Stderr, "Failed to open "+consoleDevice)
-		os.Exit(2)
-	}
-	return in
-}
-
 func (r *LightRenderer) stderr(str string) {
 	r.stderrInternal(str, true)
 }
@@ -100,11 +85,19 @@ type LightRenderer struct {
 	y             int
 	x             int
 	maxHeightFunc func(int) int
+
+	// Windows only
+	ttyinChannel    chan byte
+	inHandle        uintptr
+	outHandle       uintptr
+	origStateInput  uint32
+	origStateOutput uint32
 }
 
 type LightWindow struct {
 	renderer *LightRenderer
 	colored  bool
+	preview  bool
 	border   BorderStyle
 	top      int
 	left     int
@@ -132,10 +125,6 @@ func NewLightRenderer(theme *ColorTheme, forceBlack bool, mouse bool, tabstop in
 	return &r
 }
 
-func (r *LightRenderer) fd() int {
-	return int(r.ttyin.Fd())
-}
-
 func (r *LightRenderer) defaultTheme() *ColorTheme {
 	if strings.Contains(os.Getenv("TERM"), "256") {
 		return Dark256
@@ -147,22 +136,6 @@ func (r *LightRenderer) defaultTheme() *ColorTheme {
 	return Default16
 }
 
-func (r *LightRenderer) findOffset() (row int, col int) {
-	r.csi("6n")
-	r.flush()
-	bytes := []byte{}
-	for tries := 0; tries < offsetPollTries; tries++ {
-		bytes = r.getBytesInternal(bytes, tries > 0)
-		offsets := offsetRegexp.FindSubmatch(bytes)
-		if len(offsets) > 3 {
-			// add anything we skipped over to the input buffer
-			r.buffer = append(r.buffer, offsets[1]...)
-			return atoi(string(offsets[2]), 0) - 1, atoi(string(offsets[3]), 0) - 1
-		}
-	}
-	return -1, -1
-}
-
 func repeat(r rune, times int) string {
 	if times > 0 {
 		return strings.Repeat(string(r), times)
@@ -181,13 +154,9 @@ func atoi(s string, defaultValue int) int {
 func (r *LightRenderer) Init() {
 	r.escDelay = atoi(os.Getenv("ESCDELAY"), defaultEscDelay)
 
-	fd := r.fd()
-	origState, err := terminal.GetState(fd)
-	if err != nil {
+	if err := r.initPlatform(); err != nil {
 		errorExit(err.Error())
 	}
-	r.origState = origState
-	terminal.MakeRaw(fd)
 	r.updateTerminalSize()
 	initTheme(r.theme, r.defaultTheme(), r.forceBlack)
 
@@ -260,28 +229,6 @@ func getEnv(name string, defaultValue int) int {
 	return atoi(env, defaultValue)
 }
 
-func (r *LightRenderer) updateTerminalSize() {
-	width, height, err := terminal.GetSize(r.fd())
-	if err == nil {
-		r.width = width
-		r.height = r.maxHeightFunc(height)
-	} else {
-		r.width = getEnv("COLUMNS", defaultWidth)
-		r.height = r.maxHeightFunc(getEnv("LINES", defaultHeight))
-	}
-}
-
-func (r *LightRenderer) getch(nonblock bool) (int, bool) {
-	b := make([]byte, 1)
-	fd := r.fd()
-	util.SetNonblock(r.ttyin, nonblock)
-	_, err := util.Read(fd, b)
-	if err != nil {
-		return 0, false
-	}
-	return int(b[0]), true
-}
-
 func (r *LightRenderer) getBytes() []byte {
 	return r.getBytesInternal(r.buffer, false)
 }
@@ -316,6 +263,13 @@ func (r *LightRenderer) getBytesInternal(buffer []byte, nonblock bool) []byte {
 		}
 		buffer = append(buffer, byte(c))
 		pc = c
+
+		// This should never happen under normal conditions,
+		// so terminate fzf immediately.
+		if len(buffer) > maxInputBuffer {
+			r.Close()
+			panic(fmt.Sprintf("Input buffer overflow (%d): %v", len(buffer), buffer))
+		}
 	}
 
 	return buffer
@@ -453,7 +407,10 @@ func (r *LightRenderer) escSequence(sz *int) Event {
 			*sz = 4
 			switch r.buffer[2] {
 			case 50:
-				if len(r.buffer) == 5 && r.buffer[4] == 126 {
+				if r.buffer[3] == 126 {
+					return Event{Insert, 0, nil}
+				}
+				if len(r.buffer) > 4 && r.buffer[4] == 126 {
 					*sz = 5
 					switch r.buffer[3] {
 					case 48:
@@ -467,7 +424,7 @@ func (r *LightRenderer) escSequence(sz *int) Event {
 					}
 				}
 				// Bracketed paste mode: \e[200~ ... \e[201~
-				if r.buffer[3] == '0' && (r.buffer[4] == '0' || r.buffer[4] == '1') && r.buffer[5] == '~' {
+				if len(r.buffer) > 5 && r.buffer[3] == '0' && (r.buffer[4] == '0' || r.buffer[4] == '1') && r.buffer[5] == '~' {
 					// Immediately discard the sequence from the buffer and reread input
 					r.buffer = r.buffer[6:]
 					*sz = 0
@@ -486,10 +443,18 @@ func (r *LightRenderer) escSequence(sz *int) Event {
 				switch r.buffer[3] {
 				case 126:
 					return Event{Home, 0, nil}
-				case 53, 55, 56, 57:
+				case 49, 50, 51, 52, 53, 55, 56, 57:
 					if len(r.buffer) == 5 && r.buffer[4] == 126 {
 						*sz = 5
 						switch r.buffer[3] {
+						case 49:
+							return Event{F1, 0, nil}
+						case 50:
+							return Event{F2, 0, nil}
+						case 51:
+							return Event{F3, 0, nil}
+						case 52:
+							return Event{F4, 0, nil}
 						case 53:
 							return Event{F5, 0, nil}
 						case 55:
@@ -584,7 +549,7 @@ func (r *LightRenderer) rmcup() {
 }
 
 func (r *LightRenderer) Pause(clear bool) {
-	terminal.Restore(r.fd(), r.origState)
+	r.restoreTerminal()
 	if clear {
 		if r.fullscreen {
 			r.rmcup()
@@ -597,7 +562,7 @@ func (r *LightRenderer) Pause(clear bool) {
 }
 
 func (r *LightRenderer) Resume(clear bool) {
-	terminal.MakeRaw(r.fd())
+	r.setupTerminal()
 	if clear {
 		if r.fullscreen {
 			r.smcup()
@@ -651,7 +616,8 @@ func (r *LightRenderer) Close() {
 		r.csi("?1000l")
 	}
 	r.flush()
-	terminal.Restore(r.fd(), r.origState)
+	r.closePlatform()
+	r.restoreTerminal()
 }
 
 func (r *LightRenderer) MaxX() int {
@@ -670,6 +636,7 @@ func (r *LightRenderer) NewWindow(top int, left int, width int, height int, prev
 	w := &LightWindow{
 		renderer: r,
 		colored:  r.theme != nil,
+		preview:  preview,
 		border:   borderStyle,
 		top:      top,
 		left:     left,
@@ -693,7 +660,7 @@ func (r *LightRenderer) NewWindow(top int, left int, width int, height int, prev
 
 func (w *LightWindow) drawBorder() {
 	switch w.border.shape {
-	case BorderAround:
+	case BorderRounded, BorderSharp:
 		w.drawBorderAround()
 	case BorderHorizontal:
 		w.drawBorderHorizontal()
@@ -709,16 +676,20 @@ func (w *LightWindow) drawBorderHorizontal() {
 
 func (w *LightWindow) drawBorderAround() {
 	w.Move(0, 0)
-	w.CPrint(ColPreviewBorder, AttrRegular,
+	color := ColBorder
+	if w.preview {
+		color = ColPreviewBorder
+	}
+	w.CPrint(color, AttrRegular,
 		string(w.border.topLeft)+repeat(w.border.horizontal, w.width-2)+string(w.border.topRight))
 	for y := 1; y < w.height-1; y++ {
 		w.Move(y, 0)
-		w.CPrint(ColPreviewBorder, AttrRegular, string(w.border.vertical))
-		w.CPrint(ColPreviewBorder, AttrRegular, repeat(' ', w.width-2))
-		w.CPrint(ColPreviewBorder, AttrRegular, string(w.border.vertical))
+		w.CPrint(color, AttrRegular, string(w.border.vertical))
+		w.CPrint(color, AttrRegular, repeat(' ', w.width-2))
+		w.CPrint(color, AttrRegular, string(w.border.vertical))
 	}
 	w.Move(w.height-1, 0)
-	w.CPrint(ColPreviewBorder, AttrRegular,
+	w.CPrint(color, AttrRegular,
 		string(w.border.bottomLeft)+repeat(w.border.horizontal, w.width-2)+string(w.border.bottomRight))
 }
 
